#include "noise.h"

//Define
#define time TIME
#define saturate(x) clamp(x, 0.0, 1.0)

//Function

// Uchimura 2017, "HDR theory and practice"
// Math: https://www.desmos.com/calculator/gslcdxvipg
// Source: https://www.slideshare.net/nikuque/hdr-theory-and-practicce-jp
vec3 uchimura(vec3 x, float P, float a, float m, float l, float c, float b) {
  float l0 = ((P - m) * l) / a;
  float L0 = m - m / a;
  float L1 = m + (1.0 - m) / a;
  float S0 = m + l0;
  float S1 = m + a * l0;
  float C2 = (a * P) / (P - S1);
  float CP = -C2 / P;
  
  vec3 w0 = vec3(1.0 - smoothstep(0.0, m, x));
  vec3 w2 = vec3(step(m + l0, x));
  vec3 w1 = vec3(1.0 - w0 - w2);

  vec3 T = vec3(m * pow(x / m, vec3(c)) + b);
  vec3 S = vec3(P - (P - S1) * exp(CP * (x - S0)));
  vec3 L = vec3(m + a * (x - m));

  return T * w0 + L * w1 + S * w2;
}

vec3 uchimura(vec3 x) {
  const float P = 0.1;  // max display brightness
  const float a = 0.3; // contrast
  const float m = 0.7;  // linear section start
  const float l = 0.1;  // linear section length
  const float c = 1.0;  // black
  const float b = -0.03;  // pedestal

  return uchimura(x, P, a, m, l, c, b);
}

highp float get_wave(vec2 p, highp float time, float speed){
 return
  sin(noise(vec2(time*speed+p.s+p.s,time*speed+p.t+p.s)))
+ sin(noise(vec2(-time*speed+p.s+p.s,-time*speed+p.t+p.s)))
+ sin(noise(vec2(time*speed*2.+p.s+p.s,time*speed*2.+p.t+p.s)));
}

highp float caustic(vec2 uv, float speed){
	return voronoi(
		vec2((uv.x+uv.y+TIME*speed), (uv.y+TIME*speed)))*
		voronoi(
		vec2((uv.x-TIME*speed), (uv.y-TIME*speed)));
	}
	
highp float fbm(highp vec2 p, float density ){
highp float f = density;
  p += -TIME*0.02;
   f += 0.89*noise( p ); p = p*2.12-TIME*0.04;
 	f += -0.83*noise( p ); p = p*3.4+TIME*0.07;
 	f += 0.2*noise( p );

    	return f/0.79;
}

highp float Cloud(highp vec2 p, float den){
	float c = 0.0, b = 1.0;
for(int i = 0; i < 5; i++){
	p /= 1.04, b *= 0.89;
highp float a = smoothstep(0.0, 0.9, fbm(p.xy, den));
c = mix(c, b, a);
}
	return c;
	}